@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: alsa_driver.c,v 1.51 2004/03/27 22:08:55 joq Exp $
+    $Id: alsa_driver.c,v 1.55 2004/09/13 02:11:46 trutkin Exp $
 */
 
 #include <math.h>
@@ -30,7 +30,7 @@
 #include <sys/types.h>
 #include <regex.h>
 #include <string.h>
-
+ 
 #include <jack/internal.h>
 #include <jack/engine.h>
 
@@ -989,8 +989,7 @@ alsa_driver_start (alsa_driver_t *driver)
 
 	if (driver->playback_handle) {
 		driver->playback_nfds =
-			snd_pcm_poll_descriptors_count (
-				driver->playback_handle);
+			snd_pcm_poll_descriptors_count (driver->playback_handle);
 	} else {
 		driver->playback_nfds = 0;
 	}
@@ -1605,9 +1604,13 @@ alsa_driver_run_cycle (alsa_driver_t *driver)
 	jack_engine_t *engine = driver->engine;
 	int wait_status;
 	float delayed_usecs;
+	jack_nframes_t nframes;
 
-	jack_nframes_t nframes = alsa_driver_wait (driver, -1, &wait_status,
-						   &delayed_usecs);
+	DEBUG ("alsa run cycle wait\n");
+
+	nframes = alsa_driver_wait (driver, -1, &wait_status, &delayed_usecs);
+
+	DEBUG ("alsa back from wait, nframes = %lu", nframes);
 
 	if (nframes == 0) {
 
@@ -1938,17 +1941,30 @@ alsa_driver_new (char *name, char *playback_alsa_device,
 				  playback_alsa_device,
 				  SND_PCM_STREAM_PLAYBACK,
 				  SND_PCM_NONBLOCK) < 0) {
-			if (errno == EBUSY) {
+			switch (errno) {
+			case EBUSY:
 				jack_error ("the playback device \"%s\" is "
 					    "already in use. Please stop the"
 					    " application using it and "
 					    "run JACK again",
 					    playback_alsa_device);
 				return NULL;
-			}
+				break;
+
+			case EPERM:
+				jack_error ("you do not have permission to open "
+					    "the audio device \"%s\" for playback",
+					    playback_alsa_device);
+				return NULL;
+				break;
+			} 
+			
 			driver->playback_handle = NULL;
 		} 
-		snd_pcm_nonblock (driver->playback_handle, 0);
+
+		if (driver->playback_handle) {
+			snd_pcm_nonblock (driver->playback_handle, 0);
+		}
 	} 
 
 	if (capturing) {
@@ -1956,17 +1972,30 @@ alsa_driver_new (char *name, char *playback_alsa_device,
 				  capture_alsa_device,
 				  SND_PCM_STREAM_CAPTURE,
 				  SND_PCM_NONBLOCK) < 0) {
-			if (errno == EBUSY) {
+			switch (errno) {
+			case EBUSY:
 				jack_error ("the capture device \"%s\" is "
 					    "already in use. Please stop the"
 					    " application using it and "
 					    "run JACK again",
 					    capture_alsa_device);
 				return NULL;
-			}
+				break;
+
+			case EPERM:
+				jack_error ("you do not have permission to open "
+					    "the audio device \"%s\" for capture",
+					    capture_alsa_device);
+				return NULL;
+				break;
+			} 
+
 			driver->capture_handle = NULL;
 		}
-		snd_pcm_nonblock (driver->capture_handle, 0);
+
+		if (driver->capture_handle) {
+			snd_pcm_nonblock (driver->capture_handle, 0);
+		}
 	}
 
 	if (driver->playback_handle == NULL) {
@@ -2196,7 +2225,7 @@ driver_get_descriptor ()
 	params[i].type       = JackDriverParamString;
 	strcpy (params[i].value.str, "none");
 	strcpy (params[i].short_desc,
-		"Provide only capture ports.  Optionally set device");
+		"Provide capture ports.  Optionally set device");
 	strcpy (params[i].long_desc, params[i].short_desc);
 
 	i++;
@@ -2205,7 +2234,7 @@ driver_get_descriptor ()
 	params[i].type       = JackDriverParamString;
 	strcpy (params[i].value.str, "none");
 	strcpy (params[i].short_desc,
-		"Provide only playback ports.  Optionally set device");
+		"Provide playback ports.  Optionally set device");
 	strcpy (params[i].long_desc, params[i].short_desc);
 
 	i++;
@@ -2343,71 +2372,9 @@ driver_initialize (jack_client_t *client, const JSList * params)
 	int user_capture_nchnls = 0;
 	int user_playback_nchnls = 0;
 	int shorts_first = FALSE;
-	char *envvar;
 	const JSList * node;
 	const jack_driver_param_t * param;
 
-	/* before we do anything else, see if there are environment variables
-	   for each parameter.
-	*/
-	
-	if ((envvar = getenv ("JACK_ALSA_DEVICE")) != NULL) {
-		playback_pcm_name = envvar;
-		capture_pcm_name = envvar;
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_HWMON")) != NULL) {
-		hw_monitoring = TRUE;
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_SOFTMODE")) != NULL) {
-		soft_mode = TRUE;
-	}
-	
-	if ((envvar = getenv ("JACK_ALSA_PERIOD_FRAMES")) != NULL) {
-		frames_per_interrupt = atoi (envvar);
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_PERIODS")) != NULL) {
-		user_nperiods = atoi (envvar);
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_SRATE")) != NULL) {
-		srate = atoi (envvar);
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_DITHER")) != NULL) {
-		if (dither_opt (*envvar, &dither)) {
-			return NULL;
-		}
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_CAPTURE")) != NULL) {
-		capture = atoi (envvar);
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_PLAYBACK")) != NULL) {
-		playback = atoi (envvar);
-	}
-		
-	if ((envvar = getenv ("JACK_ALSA_CAPTURE_NCHANNELS")) != NULL) {
-		monitor = atoi (envvar);
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_PLAYBACK_NCHANNELS")) != NULL) {
-		monitor = atoi (envvar);
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_MONITOR")) != NULL) {
-		monitor = atoi (envvar);
-	}
-
-	if ((envvar = getenv ("JACK_ALSA_SHORTS_FIRST")) != NULL) {
-		shorts_first = TRUE;
-	}
-
-
-		
 	for (node = params; node; node = jack_slist_next (node)) {
   	        param = (const jack_driver_param_t *) node->data;
 
@@ -2484,7 +2451,6 @@ driver_initialize (jack_client_t *client, const JSList * params)
 			break;
 
 		}
-
 	}
 			
 	/* duplex is the default */
