@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: alsa_driver.h,v 1.4 2003/12/31 21:28:55 joq Exp $
+    $Id: alsa_driver.h,v 1.5 2004/03/25 19:31:50 joq Exp $
 */
 
 #ifndef __jack_alsa_driver_h__
@@ -36,7 +36,7 @@
 #include <jack/types.h>
 #include <jack/hardware.h>
 #include <jack/driver.h>
-#include "memops.h"
+#include <jack/memops.h>
 #include <jack/jack.h>
 
 typedef void (*ReadCopyFunction)  (jack_default_audio_sample_t *dst, char *src,
