@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: memops.c,v 1.6 2004/01/04 17:55:47 joq Exp $
+    $Id: memops.c,v 1.8 2004/03/25 19:31:50 joq Exp $
 */
 
 #define _ISOC9X_SOURCE  1
@@ -31,7 +31,7 @@
 #include <stdlib.h>
 #include <limits.h>
 
-#include "memops.h"
+#include <jack/memops.h>
 
 #define SAMPLE_MAX_24BIT  8388608.0f
 #define SAMPLE_MAX_16BIT  32768.0f
@@ -341,18 +341,18 @@ void sample_move_dither_shaped_d24_sS (char *dst,  jack_default_audio_sample_t *
 void sample_move_d16_sS (char *dst,  jack_default_audio_sample_t *src, unsigned long nsamples, unsigned long dst_skip, dither_state_t *state)
 	
 {
-	jack_default_audio_sample_t val;
+	int tmp;
 
 	/* ALERT: signed sign-extension portability !!! */
 
 	while (nsamples--) {
-		val = *src;
-		if (val > 1.0f) {
+		tmp = f_round(*src * SAMPLE_MAX_16BIT);
+		if (tmp > SHRT_MAX) {
 			*((short *)dst) = SHRT_MAX;
-		} else if (val < -1.0f) {
+		} else if (tmp < SHRT_MIN) {
 			*((short *)dst) = SHRT_MIN;
 		} else {
-			*((short *) dst) = (short) f_round(val * SAMPLE_MAX_16BIT);
+			*((short *) dst) = (short) tmp;
 		}
 		dst += dst_skip;
 		src++;
