@@ -54,6 +54,7 @@
 #include <jack/types.h>
 #include <jack/internal.h>
 #include <jack/engine.h>
+#include <jack/thread.h>
 #include <sysdeps/time.h>
 
 #include "oss_driver.h"
@@ -131,6 +132,21 @@ const static jack_driver_param_desc_t oss_params[OSS_DRIVER_N_PARAMS] = {
 /* internal functions */
 
 
+static void set_period_size (oss_driver_t *driver, 
+	jack_nframes_t new_period_size)
+{
+	driver->period_size = new_period_size;
+
+	driver->period_usecs = 
+		((double) driver->period_size /
+		(double) driver->sample_rate) * 1e6;
+	driver->last_wait_ust = 0;
+	driver->last_periodtime = jack_get_microseconds();
+	driver->next_periodtime = 0;
+	driver->iodelay = 0.0F;
+}
+
+
 static inline void update_times (oss_driver_t *driver)
 {
 	driver->last_periodtime = jack_get_microseconds();
@@ -355,6 +371,8 @@ static int oss_driver_detach (oss_driver_t *driver, jack_engine_t *engine)
 	if (driver->engine == NULL)
 		return -1;
 
+	/*jack_deactivate(driver->client);*/	/* ? */
+
 	node = driver->capture_ports;
 	while (node != NULL)
 	{
@@ -626,10 +644,12 @@ static int oss_driver_start (oss_driver_t *driver)
 	driver->threads = 0;
 	if (infd >= 0)
 	{
-		if (pthread_create(&driver->thread_in, NULL, io_thread, 
-			driver) < 0)
+		if (jack_create_thread(&driver->thread_in, 
+			driver->engine->rtpriority, 
+			driver->engine->control->real_time, 
+			io_thread, driver) < 0)
 		{
-			jack_error("OSS: pthread_create() failed: %s@%i",
+			jack_error("OSS: jack_create_thread() failed: %s@%i",
 				__FILE__, __LINE__);
 			return -1;
 		}
@@ -638,10 +658,12 @@ static int oss_driver_start (oss_driver_t *driver)
 #	ifdef USE_BARRIER
 	if (outfd >= 0)
 	{
-		if (pthread_create(&driver->thread_out, NULL, io_thread,
-			driver) < 0)
+		if (jack_create_thread(&driver->thread_out, 
+			driver->engine->rtpriority, 
+			driver->engine->control->real_time, 
+			io_thread, driver) < 0)
 		{
-			jack_error("OSS: pthread_create() failed: %s@%i",
+			jack_error("OSS: jack_create_thread() failed: %s@%i",
 				__FILE__, __LINE__);
 			return -1;
 		}
@@ -723,6 +745,7 @@ static int oss_driver_read (oss_driver_t *driver, jack_nframes_t nframes)
 	JSList *node;
 	jack_port_t *port;
 
+	if (!driver->run) return 0;
 	if (nframes != driver->period_size)
 	{
 		jack_error(
@@ -765,6 +788,7 @@ static int oss_driver_write (oss_driver_t *driver, jack_nframes_t nframes)
 	JSList *node;
 	jack_port_t *port;
 
+	if (!driver->run) return 0;
 	if (nframes != driver->period_size)
 	{
 		jack_error(
@@ -818,11 +842,9 @@ static int oss_driver_bufsize (oss_driver_t *driver, jack_nframes_t nframes)
 {
 	oss_driver_stop(driver);
 
-	driver->period_size = nframes;
-	driver->period_usecs = 
-		((double) driver->period_size / 
-		 (double) driver->sample_rate) * 1e6;
-	printf("oss_driver: period size update: %u\n", driver->period_size);
+	set_period_size(driver, nframes);
+	driver->engine->set_buffer_size(driver->engine, driver->period_size);
+	printf("oss_driver: period size update: %u\n", nframes);
 
 	oss_driver_start(driver);
 
@@ -854,26 +876,9 @@ static inline void synchronize (oss_driver_t *driver)
 
 static void *io_thread (void *param)
 {
-	int schedpol;
 	size_t localsize;
 	void *localbuf;
 	oss_driver_t *driver = (oss_driver_t *) param;
-	struct sched_param schedp;
-
-	if (pthread_getschedparam(pthread_self(), &schedpol, &schedp) == 0)
-	{
-		schedpol = SCHED_FIFO;
-		schedp.sched_priority = sched_get_priority_max(SCHED_FIFO) - 1;
-		if (pthread_setschedparam(pthread_self(), schedpol, 
-			&schedp) != 0)
-		{
-			puts("oss_driver: pthread_setschedparam() failed\n");
-		}
-	}
-	else
-	{
-		puts("oss_driver: pthread_getschedparam() failed\n");
-	}
 
 	sem_wait(&driver->sem_start);
 
@@ -1107,12 +1112,7 @@ jack_driver_t * driver_initialize (jack_client_t *client,
 	driver->capture_channels = capture_channels;
 	driver->playback_channels = playback_channels;
 
-	driver->period_usecs = 
-		((double) period_size / (double) sample_rate) * 1e6;
-	driver->last_wait_ust = 0;
-	driver->last_periodtime = jack_get_microseconds();
-	driver->next_periodtime = 0;
-	driver->iodelay = 0.0F;
+	set_period_size(driver, period_size);
 	
 	driver->finish = driver_finish;
 
