@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: engine.h,v 1.41 2004/04/12 04:14:22 pbd Exp $
+    $Id: engine.h,v 1.44 2004/08/22 21:39:53 pbd Exp $
 */
 
 #ifndef __jack_engine_h__
@@ -112,6 +112,7 @@ struct _jack_engine {
     int		    rtpriority;
     char	    freewheeling;
     char	    verbose;
+    char	    do_munlock;
     char	    temporary;
     int		    reordered;
     int		    watchdog_check;
@@ -134,6 +135,7 @@ struct _jack_engine {
     int		    rolling_client_usecs_cnt;
     int		    rolling_client_usecs_index;
     int		    rolling_interval;
+    float	    max_usecs;
     float	    spare_usecs;
     float	    usecs_per_cycle;
     
@@ -148,17 +150,17 @@ struct _jack_engine {
 /* public functions */
 
 jack_engine_t  *jack_engine_new (int real_time, int real_time_priority,
-				 int do_mlock, int temporary,
+				 int do_mlock, int do_unlock, int temporary,
 				 int verbose, int client_timeout,
 				 unsigned int port_max,
                                  pid_t waitpid, JSList *drivers);
-int             jack_engine_delete (jack_engine_t *);
-int             jack_run (jack_engine_t *engine);
-int             jack_wait (jack_engine_t *engine);
-int             jack_engine_load_driver (jack_engine_t *engine,
+void		jack_engine_delete (jack_engine_t *);
+int		jack_run (jack_engine_t *engine);
+int		jack_wait (jack_engine_t *engine);
+int		jack_engine_load_driver (jack_engine_t *engine,
 					 jack_driver_desc_t * driver_desc,
 					 JSList * driver_params);
-void            jack_dump_configuration(jack_engine_t *engine, int take_lock);
+void		jack_dump_configuration(jack_engine_t *engine, int take_lock);
 
 extern jack_client_internal_t *
 jack_client_internal_by_id (jack_engine_t *engine, jack_client_id_t id);
