@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: jack.h,v 1.63 2004/04/25 01:34:13 trutkin Exp $
+    $Id: jack.h,v 1.64 2004/07/15 03:07:28 trutkin Exp $
 */
 
 #ifndef __jack_h__
@@ -671,6 +671,16 @@ jack_nframes_t jack_frames_since_cycle_start (const jack_client_t *);
  */
 jack_nframes_t jack_frame_time (const jack_client_t *);
 
+/**
+ * @return the frame_time after the last processing of the graph
+ * this is only to be used from the process callback. 
+ *
+ * This function can be used to put timestamps generated by 
+ * jack_frame_time() in correlation to the current process cycle.
+ */
+jack_nframes_t jack_last_frame_time (const jack_client_t *client);
+
+
 /**
  * @return the current CPU load estimated by JACK.  This is a running
  * average of the time it takes to execute a full process cycle for
