@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: client.c,v 1.80 2004/04/14 03:38:32 trutkin Exp $
+    $Id: client.c,v 1.91 2004/09/15 01:33:06 joq Exp $
 */
 
 #include <config.h>
@@ -34,9 +34,7 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
-#ifdef USE_MLOCK
 #include <sys/mman.h>
-#endif /* USE_MLOCK */
 
 #include <jack/internal.h>
 #include <jack/jack.h>
@@ -45,6 +43,8 @@
 #include <jack/jslist.h>
 #include <jack/version.h>
 #include <jack/shm.h>
+#include <jack/unlock.h>
+#include <jack/thread.h>
 
 #include <sysdeps/time.h>
 JACK_TIME_GLOBAL_DECL;			/* One instance per process. */
@@ -76,8 +76,10 @@ static pthread_mutex_t client_lock;
 static pthread_cond_t  client_ready;
 void *jack_zero_filled_buffer = NULL;
 
-#define event_fd pollfd[0].fd
-#define graph_wait_fd pollfd[1].fd
+#define EVENT_POLL_INDEX 0
+#define WAIT_POLL_INDEX 1
+#define event_fd pollfd[EVENT_POLL_INDEX].fd
+#define graph_wait_fd pollfd[WAIT_POLL_INDEX].fd
 
 typedef struct {
     int status;
@@ -161,6 +163,7 @@ jack_client_alloc ()
 	client->pollmax = 2;
 	client->request_fd = -1;
 	client->event_fd = -1;
+	client->upstream_is_jackd = 0;
 	client->graph_wait_fd = -1;
 	client->graph_next_fd = -1;
 	client->ports = NULL;
@@ -300,7 +303,6 @@ jack_handle_reorder (jack_client_t *client, jack_event_t *event)
 		return -1;
 	}
 
-
 	DEBUG ("opened new graph_wait_fd %d (%s)", client->graph_wait_fd, path);
 
 	sprintf (path, "%s-%" PRIu32, client->fifo_prefix, event->x.n+1);
@@ -310,7 +312,11 @@ jack_handle_reorder (jack_client_t *client, jack_event_t *event)
 		return -1;
 	}
 
-	DEBUG ("opened new graph_next_fd %d (%s)", client->graph_next_fd, path);
+	client->upstream_is_jackd = event->y.n;
+	client->pollmax = 2;
+
+	DEBUG ("opened new graph_next_fd %d (%s) (upstream is jackd? %d)", client->graph_next_fd, path, 
+	       client->upstream_is_jackd);
 
 	/* If the client registered its own callback for graph order events,
 	   execute it now.
@@ -336,8 +342,8 @@ server_connect (int which)
 	}
 
 	addr.sun_family = AF_UNIX;
-	snprintf (addr.sun_path, sizeof (addr.sun_path) - 1, "%s/jack_%d",
-		  jack_server_dir, which);
+	snprintf (addr.sun_path, sizeof (addr.sun_path) - 1, "%s/jack_%d_%d",
+		  jack_server_dir, getuid (), which);
 
 	if (connect (fd, (struct sockaddr *) &addr, sizeof (addr)) < 0) {
 		close (fd);
@@ -362,8 +368,8 @@ server_event_connect (jack_client_t *client)
 	}
 
 	addr.sun_family = AF_UNIX;
-	snprintf (addr.sun_path, sizeof (addr.sun_path) - 1, "%s/jack_ack_0",
-		  jack_server_dir);
+	snprintf (addr.sun_path, sizeof (addr.sun_path) - 1, "%s/jack_%d_ack_0",
+		  jack_server_dir, getuid () );
 
 	if (connect (fd, (struct sockaddr *) &addr, sizeof (addr)) < 0) {
 		jack_error ("cannot connect to jack server for events",
@@ -729,8 +735,8 @@ jack_client_new (const char *client_name)
 	strcpy (client->fifo_prefix, res.fifo_prefix);
 	client->request_fd = req_fd;
 
-	client->pollfd[0].events = POLLIN|POLLERR|POLLHUP|POLLNVAL;
-	client->pollfd[1].events = POLLIN|POLLERR|POLLHUP|POLLNVAL;
+	client->pollfd[EVENT_POLL_INDEX].events = POLLIN|POLLERR|POLLHUP|POLLNVAL;
+	client->pollfd[WAIT_POLL_INDEX].events = POLLIN|POLLERR|POLLHUP|POLLNVAL;
 
 	/* attach the engine control/info block */
 	client->engine_shm = res.engine_shm;
@@ -766,6 +772,7 @@ jack_client_new (const char *client_name)
 	for (ptid = 0; ptid < client->n_port_types; ++ptid) {
 		client->port_segment[ptid].index =
 			client->engine->port_types[ptid].shm_registry_index;
+		client->port_segment[ptid].attached_at = MAP_FAILED;
 		jack_attach_port_segment (client, ptid);
 	}
 
@@ -849,59 +856,6 @@ jack_internal_client_close (const char *client_name)
 	return;
 }
 
-#if JACK_USE_MACH_THREADS 
-
-int
-jack_drop_real_time_scheduling (pthread_t thread)
-{
-	setThreadToPriority(thread, 31, false, 10000000);
-	return 0;       
-}
-
-int
-jack_acquire_real_time_scheduling (pthread_t thread, int priority) //priority is unused
-{
-	setThreadToPriority(thread, 96, true, 10000000);
-	return 0;
-}
-
-#else
-
-int
-jack_drop_real_time_scheduling (pthread_t thread)
-{
-	struct sched_param rtparam;
-	int x;
-	
-	memset (&rtparam, 0, sizeof (rtparam));
-	rtparam.sched_priority = 0;
-	
-	if ((x = pthread_setschedparam (thread, SCHED_OTHER, &rtparam)) != 0) {
-		jack_error ("cannot switch to normal scheduling priority(%s)\n", strerror (errno));
-		return -1;
-	}
-        return 0;
-}
-
-int
-jack_acquire_real_time_scheduling (pthread_t thread, int priority)
-{
-	struct sched_param rtparam;
-	int x;
-	
-	memset (&rtparam, 0, sizeof (rtparam));
-	rtparam.sched_priority = priority;
-	
-	if ((x = pthread_setschedparam (thread, SCHED_FIFO, &rtparam)) != 0) {
-		jack_error ("cannot use real-time scheduling (FIFO/%d) "
-			    "(%d: %s)", rtparam.sched_priority, x,
-			    strerror (x));
-		return -1;
-	}
-        return 0;
-}
-
-#endif
 
 int
 jack_set_freewheel (jack_client_t* client, int onoff)
@@ -956,11 +910,9 @@ jack_client_thread (void *arg)
 	jack_client_control_t *control = client->control;
 	jack_event_t event;
 	char status = 0;
-	char c;
+	char c = 0;
 	int err = 0;
 
-	pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
-
 	pthread_mutex_lock (&client_lock);
 	client->thread_ok = TRUE;
 	client->thread_id = pthread_self();
@@ -978,8 +930,8 @@ jack_client_thread (void *arg)
 	}
 
 	while (err == 0) {
-
-	        if (client->engine->engine_ok == 0) {
+	
+		if (client->engine->engine_ok == 0) {
 		     if (client->on_shutdown)
 			     client->on_shutdown (client->on_shutdown_arg);
 		     else
@@ -988,8 +940,10 @@ jack_client_thread (void *arg)
 		     pthread_exit (0);
 		}
 
-		DEBUG ("client polling on event_fd and graph_wait_fd...");
-                
+		DEBUG ("client polling on %s", client->pollmax == 2 ? 
+		       "event_fd and graph_wait_fd..." :
+		       "event_fd only");
+
 		if (poll (client->pollfd, client->pollmax, 1000) < 0) {
 			if (errno == EINTR) {
 				continue;
@@ -1004,22 +958,60 @@ jack_client_thread (void *arg)
 		 * process() cycle. 
 		 */
 
-		if (client->pollfd[1].revents & POLLIN) {
+		if (client->graph_wait_fd >= 0 && client->pollfd[WAIT_POLL_INDEX].revents & POLLIN) {
 			control->awake_at = jack_get_microseconds();
 		}
 
-		DEBUG ("pfd[0].revents = 0x%x pfd[1].revents = 0x%x",
-		       client->pollfd[0].revents,
-		       client->pollfd[1].revents);
+		DEBUG ("pfd[EVENT].revents = 0x%x pfd[WAIT].revents = 0x%x",
+		       client->pollfd[EVENT_POLL_INDEX].revents,
+		       client->pollfd[WAIT_POLL_INDEX].revents);
 
 		pthread_testcancel();
+		
+		if (client->graph_wait_fd >= 0 &&
+		    (client->pollfd[WAIT_POLL_INDEX].revents & ~POLLIN)) {
+			
+			DEBUG ("\n\n\n\n\n\n\n\nWAITFD ERROR, ZOMBIE\n\n\n\n\n");
+			
+			/* our upstream "wait" connection
+			   closed, which either means that
+			   an intermediate client exited, or
+			   jackd exited, or jackd zombified
+			   us.
 
-		if ((client->pollfd[0].revents & ~POLLIN) ||
-		    client->control->dead) {
+			   we can discover the zombification
+			   via client->control->dead, but
+			   the other two possibilities are
+			   impossible to identify just from
+			   this situation. so we have to
+			   check what we are connected to,
+			   and act accordingly.
+			*/
+			
+			if (client->upstream_is_jackd) {
+                                DEBUG ("WE DIE\n");
+				goto zombie;
+			} else {
+                                DEBUG ("WE PUNT\n");
+				/* don't poll on the wait fd
+				 * again until we get a
+				 * GraphReordered event.
+				 */
+				client->graph_wait_fd = -1;
+				client->pollmax = 1;
+			}
+		}
+
+		if (client->control->dead) {
 			goto zombie;
 		}
 
-		if (client->pollfd[0].revents & POLLIN) {
+		if (client->pollfd[EVENT_POLL_INDEX].revents & ~POLLIN) {
+			/* jackd shutdown */
+			goto zombie;
+		}
+
+		if (client->pollfd[EVENT_POLL_INDEX].revents & POLLIN) {
 
 			DEBUG ("client receives an event, "
 			       "now reading on event fd");
@@ -1114,11 +1106,7 @@ jack_client_thread (void *arg)
 			}
 		}
 
-		if (client->pollfd[1].revents & ~POLLIN) {
-			goto zombie;
-		}
-
-		if (client->pollfd[1].revents & POLLIN) {
+		if (client->pollfd[WAIT_POLL_INDEX].revents & POLLIN) {
 
 #ifdef WITH_TIMESTAMPS
 			jack_reset_timestamps ();
@@ -1132,7 +1120,7 @@ jack_client_thread (void *arg)
 			       control->signalled_at, 
 			       control->awake_at, 
 			       control->awake_at - control->signalled_at,
-			       client->pollfd[1].fd);
+			       client->pollfd[WAIT_POLL_INDEX].fd);
 
 			control->state = Running;
 
@@ -1246,12 +1234,10 @@ jack_client_process_thread (void *arg)
 	client->control->pid = getpid();
 	DEBUG ("client process thread is now running");
         
-	pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
-        
 	client->rt_thread_ok = TRUE;
             
    	while (err == 0) {
-        
+	
                 if (jack_client_suspend(client) < 0) {
                         jack_error ("jack_client_process_thread : resume error");
                         goto zombie;
@@ -1321,182 +1307,65 @@ jack_client_process_thread (void *arg)
 
 static int
 jack_start_thread (jack_client_t *client)
-
 {
-#ifndef JACK_USE_MACH_THREADS
-	int policy = SCHED_OTHER;
-	struct sched_param client_param, temp_param;
-#endif
-	pthread_attr_t *attributes = 0;
 
-	if (client->engine->real_time) {
-		
-		/* Get the client thread to run as an RT-FIFO
-		   scheduled thread of appropriate priority.
-		*/
+ 	if (client->engine->real_time) {
 
-		struct sched_param rt_param;
-
-		attributes = (pthread_attr_t *)
-			malloc (sizeof (pthread_attr_t));
-
-		pthread_attr_init (attributes);
-
-		if (pthread_attr_setschedpolicy (attributes, SCHED_FIFO)) {
-			jack_error ("cannot set FIFO scheduling class for RT "
-				    "thread");
-			return -1;
-		}
-
-		if (pthread_attr_setscope (attributes, PTHREAD_SCOPE_SYSTEM)) {
-			jack_error ("Cannot set scheduling scope for RT "
-				    "thread");
-			return -1;
-		}
-
-		memset (&rt_param, 0, sizeof (rt_param));
-		rt_param.sched_priority = client->engine->client_priority;
-
-		if (pthread_attr_setschedparam (attributes, &rt_param)) {
-			jack_error ("Cannot set scheduling priority for RT "
-				    "thread (%s)", strerror (errno));
-			return -1;
-		}
-                
 #ifdef USE_MLOCK
-                if (client->engine->do_mlock
-		    && (mlockall (MCL_CURRENT | MCL_FUTURE) != 0)) {
-                    jack_error ("cannot lock down memory for RT thread (%s)",
-				strerror (errno));
+		 if (client->engine->do_mlock
+		     && (mlockall (MCL_CURRENT | MCL_FUTURE) != 0)) {
+			 jack_error ("cannot lock down memory for RT thread (%s)",
+				     strerror (errno));
+			 
 #ifdef ENSURE_MLOCK
-                    return -1;
+			 return -1;
 #endif /* ENSURE_MLOCK */
-                }
+		 }
+		 
+		 if (client->engine->do_munlock) {
+			 cleanup_mlock ();
+		 }
 #endif /* USE_MLOCK */
 	}
 
 #ifdef JACK_USE_MACH_THREADS
-
-	if (pthread_create (&client->thread, attributes,
-			    jack_client_thread, client)) {
+/* Stephane Letz : letz@grame.fr
+On MacOSX, the normal thread does not need to be real-time.
+*/
+	if (jack_create_thread (&client->thread,
+				client->engine->client_priority,
+				0,
+				jack_client_thread, client)) {
+		return -1;
+	}
+#else
+	if (jack_create_thread (&client->thread,
+				client->engine->client_priority,
+				client->engine->real_time,
+				jack_client_thread, client)) {
 		return -1;
 	}
 
-        /* a special real-time thread to call the "process"
-	 * callback. It will communicate with the server using fast
-	 * mach RPC mechanism */
-        if (pthread_create (&client->process_thread, attributes,
-			    jack_client_process_thread, client)) {
-                jack_error("pthread_create failed for process_thread \n");
-                return -1;
-        }
-        if (client->engine->real_time){
-            /* time constraint thread */
-            setThreadToPriority(client->process_thread, 96, TRUE, 10000000);
-        }else{
-            /* fixed priority thread */
-            setThreadToPriority(client->process_thread, 63, TRUE, 10000000);
-        }
+#endif
+	
+#ifdef JACK_USE_MACH_THREADS
 
-#else /* !JACK_USE_MACH_THREADS */
+	/* a secondary thread that runs the process callback and uses
+	   ultra-fast Mach primitives for inter-thread signalling.
 
-	if (pthread_create (&client->thread, attributes,
-			    jack_client_thread, client) == 0) {
-		return 0;
-	}
+	   XXX in a properly structured JACK, there would be no
+	   need for this, because we would have client wake up
+	   methods that encapsulated the underlying mechanism
+	   used.
 
-	if (!client->engine->real_time) {
-		return -1;
-	}
-
-	/* the version of glibc I've played with has a bug that makes
-	   that code fail when running under a non-root user but with the
-	   proper realtime capabilities (in short,  pthread_attr_setschedpolicy 
-	   does not check for capabilities, only for the uid being
-	   zero). Newer versions apparently have this fixed. This
-	   workaround temporarily switches the client thread to the
-	   proper scheduler and priority, then starts the realtime
-	   thread so that it can inherit them and finally switches the
-	   client thread back to what it was before. Sigh. For ardour
-	   I have to check again and switch the thread explicitly to
-	   realtime, don't know why or how to debug - nando
 	*/
 
-	/* get current scheduler and parameters of the client process */
-	if ((policy = sched_getscheduler (0)) < 0) {
-		jack_error ("Cannot get current client scheduler: %s",
-			    strerror(errno));
+	if (jack_create_thread(&client->process_thread,
+				client->engine->client_priority,
+				client->engine->real_time,
+				jack_client_process_thread, client)) {
 		return -1;
 	}
-	memset (&client_param, 0, sizeof (client_param));
-	if (sched_getparam (0, &client_param)) {
-		jack_error ("Cannot get current client scheduler "
-			    "parameters: %s", strerror(errno));
-		return -1;
-	}
-
-	/* temporarily change the client process to SCHED_FIFO so that
-	   the realtime thread can inherit the scheduler and priority
-	*/
-	memset (&temp_param, 0, sizeof (temp_param));
-	temp_param.sched_priority = client->engine->client_priority;
-	if (sched_setscheduler(0, SCHED_FIFO, &temp_param)) {
-		jack_error ("Cannot temporarily set client to RT scheduler:"
-			    " %s", strerror(errno));
-		return -1;
-	}
-
-	/* prepare the attributes for the realtime thread */
-	attributes = (pthread_attr_t *) malloc (sizeof (pthread_attr_t));
-	pthread_attr_init (attributes);
-	if (pthread_attr_setscope (attributes, PTHREAD_SCOPE_SYSTEM)) {
-		sched_setscheduler (0, policy, &client_param);
-		jack_error ("Cannot set scheduling scope for RT thread");
-		return -1;
-	}
-	if (pthread_attr_setinheritsched (attributes, PTHREAD_INHERIT_SCHED)) {
-		sched_setscheduler (0, policy, &client_param);
-		jack_error ("Cannot set scheduler inherit policy for RT "
-			    "thread");
-		return -1;
-	}
-
-	/* create the RT thread */
-	if (pthread_create (&client->thread, attributes,
-			    jack_client_thread, client)) {
-		sched_setscheduler (0, policy, &client_param);
-		return -1;
-	}
-
-	/* return the client process to the scheduler it was in before */
-	if (sched_setscheduler (0, policy, &client_param)) {
-		jack_error ("Cannot reset original client scheduler: %s",
-			    strerror(errno));
-		return -1;
-	}
-
-	/* check again... inheritance of policy and priority works in
-	   jack_simple_client but not in ardour! So I check again and
-	   force the policy if it is not set correctly. This does not
-	   really really work either, the manager thread of the
-	   linuxthreads implementation is left running with
-	   SCHED_OTHER, that is presumably very bad.
-	*/
-	memset (&client_param, 0, sizeof (client_param));
-	if (pthread_getschedparam(client->thread, &policy,
-				  &client_param) == 0) {
-		if (policy != SCHED_FIFO) {
-			memset (&client_param, 0, sizeof (client_param));
-			client_param.sched_priority =
-				client->engine->client_priority;
-			if (pthread_setschedparam (client->thread, SCHED_FIFO,
-						   &client_param)) {
-				jack_error ("Cannot set (again) FIFO scheduling"
-					    " class for RT thread\n");
-				return -1;
-			}
-		}
-	}
 #endif /* JACK_USE_MACH_THREADS */
 
 	return 0;
@@ -1854,18 +1723,6 @@ jack_set_port_registration_callback(jack_client_t *client,
 	return 0;
 }
 
-int
-jack_get_process_start_fd (jack_client_t *client)
-{
-	/* once this has been called, the client thread
-	   does not sleep on the graph wait fd.
-	*/
-
-	client->pollmax = 1;
-	return client->graph_wait_fd;
-
-}
-
 int
 jack_get_process_done_fd (jack_client_t *client)
 {
@@ -1943,6 +1800,12 @@ jack_get_ports (jack_client_t *client,
 			matching_ports[match_cnt++] = psp[i].name;
 		}
 	}
+	if (port_name_pattern && port_name_pattern[0]) {
+		regfree (&port_regex);
+	}
+	if (type_name_pattern && type_name_pattern[0]) {
+		regfree (&type_regex);
+	}
 
 	matching_ports[match_cnt] = 0;
 
