@@ -15,19 +15,29 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
     
-    $Id: pool.c,v 1.4 2003/09/26 17:08:43 joq Exp $
+    $Id: pool.c,v 1.6 2004/03/25 19:31:51 joq Exp $
 */
 
+#include <config.h>
+
+#ifdef HAVE_POSIX_MEMALIGN
+#define _XOPEN_SOURCE 600
+#endif
 #include <stdlib.h>
 #include <config.h>
 #include <jack/pool.h>
 
+/* XXX need RT-pool based allocator here */
 void *
 jack_pool_alloc (size_t bytes)
 {
-	/* XXX need RT-pool based allocator here */
-
+#ifdef HAVE_POSIX_MEMALIGN
+	void* m;
+	int	err = posix_memalign (&m, 16, bytes);
+	return (!err) ? m : 0;
+#else
 	return malloc (bytes);
+#endif /* HAVE_POSIX_MEMALIGN */
 }
 
 void
