@@ -15,8 +15,9 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: shm.c,v 1.13 2004/01/08 19:58:08 joq Exp $
+    $Id: shm.c,v 1.19 2004/04/12 04:14:22 pbd Exp $
 */
+#include <config.h>
 
 #include <unistd.h>
 #include <fcntl.h>
@@ -25,12 +26,10 @@
 #include <signal.h>
 #include <limits.h>
 #include <errno.h>
+#include <dirent.h>
 #include <sys/mman.h>
 #include <sys/types.h>
-#include <dirent.h>
-#include <sys/ipc.h>
 #include <sys/shm.h>
-#include <config.h>
 
 #include <jack/shm.h>
 #include <jack/internal.h>
@@ -101,21 +100,18 @@ jack_cleanup_shm (void)
 		destroy = FALSE;
 
 		if (r->allocator == getpid()) {
-			
+
 			/* allocated by this process, so unattach 
-			   and destroy.
-			*/
-			
+			   and destroy. */
 			jack_release_shm (&copy);
 			destroy = TRUE;
-			
+
 		} else {
-			
+
 			if (kill (r->allocator, 0)) {
 				if (errno == ESRCH) {
-					
-					/* allocator no longer exists, so destroy */
-					
+					/* allocator no longer exists,
+					 * so destroy */
 					destroy = TRUE;
 				}
 			}
@@ -123,8 +119,9 @@ jack_cleanup_shm (void)
 		
 		if (destroy) {
 
-			jack_destroy_shm (&copy);
-			
+			if (copy.index >= 0  && copy.index < MAX_SHM_ID) {
+				jack_destroy_shm (&copy);
+			}
 			r->size = 0;
 			r->allocator = 0;
 		}
@@ -133,7 +130,7 @@ jack_cleanup_shm (void)
 	jack_shm_unlock_registry ();
 }
 
-#if USE_POSIX_SHM
+#ifdef USE_POSIX_SHM
 
 int
 jack_initialize_shm (void)
@@ -142,6 +139,7 @@ jack_initialize_shm (void)
 	jack_shmsize_t size;
 	int new_registry = FALSE;
 	int ret = -1;
+	int perm;
 
 	if (jack_shm_registry != NULL) {
 		return 0;
@@ -149,43 +147,44 @@ jack_initialize_shm (void)
 
 	/* grab a chunk of memory to store shm ids in. this is 
 	   to allow clean up of all segments whenever JACK
-	   starts (or stops).
-	*/
-
+	   starts (or stops). */
 	size = sizeof (jack_shm_registry_t) * MAX_SHM_ID;
-
 	jack_shm_lock_registry ();
+	perm = O_RDWR;
 
 	/* try without O_CREAT to see if it already exists */
-
-	if ((shm_fd = shm_open ("/jack-shm-registry", O_RDWR, 0666)) < 0) {
+	if ((shm_fd = shm_open ("/jack-shm-registry", perm, 0666)) < 0) {
 
 		if (errno == ENOENT) {
 			
 			/* it doesn't exist, so create it */
-
-			if ((shm_fd = shm_open ("/jack-shm-registry", O_RDWR|O_CREAT, 0666)) < 0) {
-				jack_error ("cannot create shm registry segment (%s)",
-					    strerror (errno));
+			perm = O_RDWR|O_CREAT;
+			if ((shm_fd =
+			     shm_open ("/jack-shm-registry", perm, 0666)) < 0) {
+				jack_error ("cannot create shm registry segment"
+					    " (%s)", strerror (errno));
 				goto out;
 			}
 			new_registry = TRUE;
 
 		} else {
 
-			jack_error ("cannot open existing shm registry segment (%s)",
-				    strerror (errno));
+			jack_error ("cannot open existing shm registry segment"
+				    " (%s)", strerror (errno));
 			goto out;
 		}
 	}
 
-	if (ftruncate (shm_fd, size) < 0) {
-		jack_error ("cannot set size of engine shm registry "
-			    "(%s)", strerror (errno));
-		goto out;
+	if (perm & O_CREAT) {
+		if (ftruncate (shm_fd, size) < 0) {
+			jack_error ("cannot set size of engine shm registry 1"
+			"(%s)", strerror (errno));
+			goto out;
+		}
 	}
-
-	if ((jack_shm_registry = mmap (0, size, PROT_READ|PROT_WRITE, MAP_SHARED, shm_fd, 0)) == MAP_FAILED) {
+  
+	if ((jack_shm_registry = mmap (0, size, PROT_READ|PROT_WRITE,
+				       MAP_SHARED, shm_fd, 0)) == MAP_FAILED) {
 		jack_error ("cannot mmap shm registry segment (%s)",
 			    strerror (errno));
 		goto out;
@@ -229,6 +228,7 @@ jack_shmalloc (const char *shm_name, jack_shmsize_t size, jack_shm_info_t* si)
 {
 	jack_shm_registry_t* registry;
 	int shm_fd;
+	int perm = O_RDWR|O_CREAT;
 
 	if ((registry = jack_get_free_shm_info ()) == NULL) {
 		return -1;
@@ -239,11 +239,13 @@ jack_shmalloc (const char *shm_name, jack_shmsize_t size, jack_shm_info_t* si)
 			    strerror (errno));
 		return -1;
 	}
-
-	if (ftruncate (shm_fd, size) < 0) {
-		jack_error ("cannot set size of engine shm registry "
-			    "(%s)", strerror (errno));
-		return -1;
+        
+	if (perm & O_CREAT) {
+		if (ftruncate (shm_fd, size) < 0) {
+				jack_error ("cannot set size of engine shm "
+					    "registry 0 (%s)", strerror (errno));
+				return -1;
+		}
 	}
 
 	close (shm_fd);
@@ -306,8 +308,7 @@ jack_resize_shm (jack_shm_info_t* si, jack_shmsize_t size)
 	}
 		
 	if ((si->attached_at = mmap (0, size, PROT_READ|PROT_WRITE,
-				     MAP_SHARED, shm_fd, 0))
-	    == MAP_FAILED) {
+				     MAP_SHARED, shm_fd, 0)) == MAP_FAILED) {
 		jack_error ("cannot mmap shm segment %s (%s)", registry->id,
 			    strerror (errno));
 		close (shm_fd);
@@ -352,18 +353,18 @@ jack_initialize_shm (void)
 
 	if ((shmid = shmget (key, size, shmflags)) < 0) {
 		if (errno == ENOENT) {
-			if ((shmid = shmget (key, size, shmflags|IPC_CREAT)) < 0) {
-				jack_error ("cannot create shm registry segment (%s)",
-					    strerror (errno));
+			if ((shmid = shmget (key, size,
+					     shmflags|IPC_CREAT)) < 0) {
+				jack_error ("cannot create shm registry segment"
+					    " (%s)", strerror (errno));
 				goto out;
 			}
-			
 			new_registry = TRUE;
 			
 		} else {
 
-			jack_error ("cannot use existing shm registry segment (%s)",
-				    strerror (errno));
+			jack_error ("cannot use existing shm registry segment"
+				    " (%s)", strerror (errno));
 			goto out;
 		}
 	}
@@ -406,7 +407,8 @@ jack_release_shm (jack_shm_info_t* si)
 }
 
 int
-jack_shmalloc (const char* name_not_used, jack_shmsize_t size, jack_shm_info_t* si) 
+jack_shmalloc (const char* name_not_used, jack_shmsize_t size,
+	       jack_shm_info_t* si) 
 {
 	int shmflags;
 	int shmid;
@@ -437,7 +439,8 @@ jack_shmalloc (const char* name_not_used, jack_shmsize_t size, jack_shm_info_t*
 int
 jack_attach_shm (jack_shm_info_t* si)
 {
-	if ((si->attached_at = shmat (jack_shm_registry[si->index].id, 0, 0)) < 0) {
+	if ((si->attached_at = shmat (jack_shm_registry[si->index].id,
+				      0, 0)) < 0) {
 		jack_error ("cannot attach shm segment (%s)",
 			    strerror (errno));
 		jack_release_shm_info (si->index);
